package sample.bloommaker_process_and_task_form;

import jp.co.intra_mart.activiti.engine.ActivitiException;
import jp.co.intra_mart.activiti.engine.ActivitiIllegalArgumentException;
import jp.co.intra_mart.activiti.engine.delegate.DelegateExecution;
import jp.co.intra_mart.activiti.engine.delegate.ExecutionListener;
import jp.co.intra_mart.common.aid.jdk.java.lang.StringUtil;
import jp.co.intra_mart.common.platform.log.Logger;
import jp.co.intra_mart.foundation.repository.RepositoryServiceProvider;
import jp.co.intra_mart.foundation.repository.ValidateResult;
import jp.co.intra_mart.foundation.repository.metadata.dictionary.Dictionary;
import jp.co.intra_mart.foundation.repository.metadata.dictionary.Item;
import jp.co.intra_mart.foundation.repository.metadata.dictionary.ItemId;
import jp.co.intra_mart.foundation.repository.metadata.dictionary.exception.DictionaryServiceException;
import jp.co.intra_mart.foundation.repository.metadata.dictionary.restriction.Restriction;
import jp.co.intra_mart.foundation.repository.metadata.dictionary.usage.UsageId;
import jp.co.intra_mart.foundation.repository.metadata.dictionary.usage.Usages;
import jp.co.intra_mart.foundation.security.exception.AccessSecurityException;
import jp.co.intra_mart.foundation.security.message.MessageManager;
import jp.co.intra_mart.system.repository.metadata.dictionary.restriction.DigitRestriction;
import jp.co.intra_mart.system.repository.metadata.dictionary.usage.RestrictionUsage;

public class BPMProcessStartValidationListener implements ExecutionListener {

    private static final long serialVersionUID = -5613803000131436455L;

    private static Logger logger = Logger.getLogger(BPMProcessStartValidationListener.class);

    @Override
    public void notify(DelegateExecution execution) throws Exception {
        final MessageManager messageManager = MessageManager.getInstance();
        // 画面入力値由来の変数の文字数や必須項目の有無の検証を行います。
        final String businessKey = execution.getProcessBusinessKey();
        final String assignee = (String) execution.getVariable("assignee");
        final String description = (String) execution.getVariable("description");

        boolean isInvalid = false;
        try {
            // IM-Repositoryの辞書項目を使用してバリデーションを行います。
            isInvalid = !validateDigitRestriction("bloommaker_process_and_task_form_business_key", "businessKey", businessKey, "E.BPM.SAMPLE.0001");

            if (StringUtil.isBlank(assignee)) {
                isInvalid = true;
                if (logger.isErrorEnabled()) {
                    logger.error(messageManager.getMessage("E.BPM.SAMPLE.0002"));
                }
            }

            // IM-Repositoryの辞書項目を使用してバリデーションを行います。
            isInvalid = !validateDigitRestriction("bloommaker_process_and_task_form_description", "description", description, "E.BPM.SAMPLE.0003");

            if (isInvalid) {
                throw new ActivitiIllegalArgumentException(messageManager.getMessage("E.BPM.SAMPLE.0005"));
            }
        } catch (final AccessSecurityException e) {
            // messageManager#getMessage
            // 検査例外「アクセスセキュリティ例外クラス（AccessSecurityException）」を補足した場合、
            // ActivitiExceptionでラップしスローします。
            throw new ActivitiException(e.getMessage(), e);
        }
    }

    public boolean validateDigitRestriction(final String itemId, String name, final String target, final String msgCd) throws AccessSecurityException {
        if (target == null) {
            return true;
        }
        MessageManager messageManager = MessageManager.getInstance();
        try {
            final Dictionary dictionary = RepositoryServiceProvider.getInstance().getDictionaryService().getCurrent();

            final Item item = dictionary.find(ItemId.of(itemId));
            if (item != null) {
                final Usages usages = item.getUsages();
                final RestrictionUsage restrictionUsage = usages.get(UsageId.of(RestrictionUsage.ID));
                for (final Restriction restriction : restrictionUsage.getRestrictions()) {
                    if (restriction instanceof DigitRestriction) {
                        final ValidateResult validationResult = restriction.validate(name, target);
                        if (validationResult.getInvalidParams().size() > 0) {
                            if (logger.isErrorEnabled()) {
                                logger.error(messageManager.getMessage(msgCd, ((DigitRestriction) restriction).getMax()));
                            }
                            return false;
                        }
                    }
                }
            } else {
                throw new ActivitiException(messageManager.getMessage("MSG.E.BPM.SAMPLE.ERROR.EXCEPTION"));
            }
            return true;
        } catch (DictionaryServiceException e) {
            throw new ActivitiIllegalArgumentException(messageManager.getMessage("MSG.E.BPM.SAMPLE.ERROR.EXCEPTION"), e);
        }
    }
}
